#!/usr/bin/env python
# vim: set fileencoding=utf-8 :

import numpy
from bob.pad.base.algorithm import Algorithm

import bob.io.base
from sklearn import mixture

class SKLGMM(Algorithm):
  """
  This class interfaces a GMM-based "classifier" to perform PAD experiments

  A GMM is used to model the bonafide features

  """

  def __init__(self, n_gaussians=2, max_iter=1000, conv_threshold=1e-5, **kwargs):

    Algorithm.__init__(self,  
                       performs_projection=True,
                       requires_projector_training=True, 
                       **kwargs)

    self.n_gaussians = n_gaussians
    self.max_iter = max_iter
    self.conv_threshold = conv_threshold

    self.machine = mixture.GaussianMixture(n_components=n_gaussians, tol=conv_threshold, max_iter=max_iter)
    self.parameters_keys = [ "covariance_type", "covariances_", "lower_bound_", "means_",
                             "n_components", "weights_", "converged_", "precisions_", "precisions_cholesky_"]


  def train_projector(self, training_features, projector_file):
    """
    Trains the GMM using Expectation-Maximimazation with Maximum Likelihood criterion 

    **Parameters**

    training_features:
    """
    # training_features[0] - training features for the REAL class.
    # training_features[1] - training features for the ATTACK class.
    
    # The data - "positive class only"
    pos = numpy.array(training_features[0])
    features_dim = pos.shape[1]
  
    # train
    self.machine.fit(pos)

    # save
    f = bob.io.base.HDF5File(projector_file, 'w')
    for key in self.parameters_keys:
      data = getattr(self.machine, key)
      f.set(key, data)
    
  def load_projector(self, projector_file):
     
    f = bob.io.base.HDF5File(projector_file, 'r')  # file to read the machine from
    self.machine = mixture.GaussianMixture()

    for key in self.parameters_keys:
      data = f.read(key)
      setattr(self.machine, key, data)

  def project(self, feature):
    """
    Compute the log-likelihood of the feature 
    """
    # sklearn.mixture.GaussianMixture expects a 2d array (even with one example)
    feature = numpy.expand_dims(feature, axis=0)
    return self.machine.score_samples(feature)

  def score(self, toscore):
    return [toscore[0]]
